/*
Dozen Days of Words episode 9: Crystal Skull
Copyright (c) 2009 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.words.skull
{
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Rectangle;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import com.blazinggames.dozendays.RadioToggle;
	
	public class SpookyText
	extends MovieClip
	{
// ***** CONSTANTS **********************************************************

		private static const DEFAULT_MIN_SIZE:int = 8;
		private static const DEFAULT_MAX_SIZE:int = 100;
		private static const DEFAULT_LOW_COLOR:uint = 0x808080;
		private static const DEFAULT_HIGH_COLOR:uint = 0xFFFFFF;
/*
		private static const INC_SIZE:int = 0;
		private static const DEC_SIZE:int = 1;
		private static const INC_RED:int = 2;
		private static const DEC_RED:int = 3;
		private static const INC_GREEN:int = 4;
		private static const DEC_GREEN:int = 5;
		private static const INC_BLUE:int = 6;
		private static const DEC_BLUE:int = 7;
		private static const NUM_ACTIONS:int = 8;
*/
// ***** VARIABLES **********************************************************

		// range holders
		protected var _lowSize:int;
		protected var _highSize:int;
		protected var _lowRed:int;
		protected var _highRed:int;
		protected var _lowGreen:int;
		protected var _highGreen:int;
		protected var _lowBlue:int;
		protected var _highBlue:int;
		
		// current state
		public var _message:String
		public var _format:TextFormat;
		protected var _size:int;
		protected var _red:uint;
		protected var _green:uint;
		protected var _blue:uint;
		
		// target state
		protected var _targetSize:int;
		protected var _targetRed:uint;
		protected var _targetGreen:uint;
		protected var _targetBlue:uint;
		
		// "visible" components
		protected var _text:TextField;

// ***** CONSTRUCTOR ********************************************************

		public function SpookyText(
				w:int, 
				h:int, 
				s:String, 
				minSize:int = DEFAULT_MIN_SIZE,
				maxSize:int = DEFAULT_MAX_SIZE,
				lowColor:uint = DEFAULT_LOW_COLOR,
				highColor:uint = DEFAULT_HIGH_COLOR
		)
		{
			setSizeRange(minSize, maxSize);
			setColorRange(lowColor, highColor);
			_message = s;
			_size = (_lowSize + _highSize) / 2;
			_red = (_lowRed + _highRed) / 2;
			_green = (_lowGreen + _highGreen) / 2;
			_blue = (_lowBlue + _highBlue) / 2;
			_targetSize = _size;
			_targetRed = _red;
			_targetGreen = _green;
			_targetBlue = _blue;
			var curColor:uint = (_red << 16) | (_green << 8) | _blue;
			_format = new TextFormat(null, _size, curColor, null, null, null, null, null, TextFormatAlign.CENTER);
			_text = new TextField();
			_text.width = w;
			_text.height = h;
			_text.defaultTextFormat = _format;
			_text.text = _message;
			addChild(_text)
			addEventListener(Event.ENTER_FRAME, handleEnterFrame);
		}

		public function handleEnterFrame(evt:Event):void
		{
			if (_size == _targetSize) {
				_targetSize = Math.random() * (_highSize - _lowSize+1) + _lowSize;
			} else if (_size < _targetSize) {
				++_size;
			} else {
				--_size;
			}

			if (_red == _targetRed) {
				_targetRed = Math.random() * (_highRed - _lowRed+1) + _lowRed;
			} else if (_red < _targetRed) {
				++_red;
			} else {
				--_red;
			}

			if (_green == _targetGreen) {
				_targetGreen = Math.random() * (_highGreen - _lowGreen+1) + _lowGreen;
			} else if (_green < _targetGreen) {
				++_green;
			} else {
				--_green;
			}

			if (_blue == _targetBlue) {
				_targetBlue = Math.random() * (_highBlue - _lowBlue+1) + _lowBlue;
			} else if (_blue < _targetBlue) {
				++_blue;
			} else {
				--_blue;
			}

/*
			var action:int = Math.random() * NUM_ACTIONS;
			var actionTaken:Boolean;			
			do {
				actionTaken = true;
				action = (action + 1) % NUM_ACTIONS;
				switch (action) {
				case INC_SIZE:
					if (_size >= _highSize)
						actionTaken = false;
					else
						++_size;
					break;
				case DEC_SIZE:
					if (_size <= _lowSize)
						actionTaken = false;
					else
						--_size;
					break;
				case INC_RED:
					if (_red >= _highRed)
						actionTaken = false;
					else
						++_red;
					break;				
				case DEC_RED:
					if (_red <= _lowRed)
						actionTaken = false;
					else
						--_red;
					break;
				case INC_GREEN:
					if (_green >= _highGreen)
						actionTaken = false;
					else
						++_green;
					break;

				case DEC_GREEN:
					if (_green <= _lowGreen)
						actionTaken = false;
					else
						--_green;
					break;
				case INC_BLUE:
					if (_blue >= _highBlue)
						actionTaken = false;
					else
						++_blue;
					break;
				case DEC_BLUE:
					if (_blue <= _lowBlue)
						actionTaken = false;
					else
						--_blue;
					break;
				}
			} while ( ! actionTaken);
*/
			_format.size = _size;
			var curColor:uint = (_red << 16) | (_green << 8) | _blue;
			_format.color = curColor;
			_text.defaultTextFormat = _format;
			_text.text = _message;
		}
		
// ***** Functions - Getting and Setting**************************************

		public function setSizeRange(low:int, high:int):void
		{
			_lowSize = Math.min(low, high);
			_highSize = Math.max(low, high);
		}
		
		public function setColorRange(low:uint, high:uint):void
		{
			var r1:uint = (low & 0xFF0000) >> 16;
			var r2:uint = (high & 0xFF0000) >> 16;
			var g1:uint = (low & 0xFF00) >> 8;
			var g2:uint = (high & 0xFF00) >> 8;
			var b1:uint = low & 0xFF;
			var b2:uint = high & 0xFF;
			
			_lowRed = Math.min(r1, r2);
			_highRed = Math.max(r1, r2);
			_lowGreen = Math.min(g1, g2);
			_highGreen = Math.max(g1, g2);
			_lowBlue = Math.min(b1, b2);
			_highBlue = Math.max(b1, b2);
			
		}
		
		public function setText(s:String):void
		{
			_message = s;
			_text.text = s;
		}
	}
}

